#include "StdAfx.h"
#include <math.h>
#include "mm.h"
#include "MatrixAlgebra.h"
	
void DrawProbDistribution (CDC &dc, int offset_x, int offset_y, dMatrix &C)
{
	// Draw a probability distribution
	// wikipedia Multivariate normal distribution 

	dMatrix Cinv = !C;

	for (double x=-100; x<100; x+=1) {
		for (double y=-100; y<100; y+=1) {
			/*
			dMatrix v(2,1);
			v(0,0) = x;
			v(1,0) = y;

			dMatrix e = ~v*Cinv*v;
			double p = exp(-0.5*e(0,0));
			*/

			//  ּ ó   Ǯ  .
			double p = exp (-0.5*(x*x*Cinv(0,0) + 2*x*y*Cinv(0,1) + y*y*Cinv(1,1)));

			dc.SetPixel ((int)(offset_x + x), (int)(offset_y - y), RGB(255*p,0,0));
		}
	}
}

void DrawCovarianceEllipse_Rotat (CDC &dc, int offset_x, int offset_y, double sigma_xx, double sigma_yy, double sigma_xy)
{
	// Draw covariance ellipse
	// Derivation from Rotations
	// http://www.geom.unimelb.edu.au/nicole/surveynetworks/02a/notes09_01.html 

	double theta = 0.5*atan(2*sigma_xy/(sigma_yy - sigma_xx));

	double st = sin(theta);
	double ct = cos(theta);

	double a = sqrt(sigma_xx*ct*ct - 2*sigma_xy*st*ct + sigma_yy*st*st);
	double b = sqrt(sigma_xx*st*st + 2*sigma_xy*st*ct + sigma_yy*ct*ct);

	for (int i=0; i<=360; i+=10) {
		double ri = _DEG2RAD*i;

		double u = a*cos(ri);
		double v = b*sin(ri);

		double rx =  u*ct + v*st;
		double ry = -u*st + v*ct;

		if (i==0) dc.MoveTo ((int)(offset_x + rx), (int)(offset_y - ry));
		else      dc.LineTo ((int)(offset_x + rx), (int)(offset_y - ry));
	}
}

void DrawCovarianceEllipse_Eigen (CDC &dc, int offset_x, int offset_y, dMatrix &C)
{
	// Draw covariance ellipse
	// Derivation from Eigen value and vectors
	// Paul Michael Newman "C4B | Mobile Robotics" Topic 10.1 Drawing Covariance Ellipses 

	dVector D;
	dMatrix V;

	if (C.eigen(D, V)) {
		for (unsigned int i = 0; i<D.size(); ++i) {
			D[i] = sqrt(D[i]);
		}

		for (int i=0; i<=360; i+=10) {
			double ri = _DEG2RAD*i;

			double x = D[0]*cos(ri);
			double y = D[1]*sin(ri);

			double rx = V(0,0)*x + V(0,1)*y;
			double ry = V(1,0)*x + V(1,1)*y;

			if (i==0) dc.MoveTo ((int)(offset_x + rx), (int)(offset_y - ry));
			else      dc.LineTo ((int)(offset_x + rx), (int)(offset_y - ry));
		}
	}
}

void DrawParticles (CDC &dc, int offset_x, int offset_y, dMatrix &C)
{
	dVector D;
	dMatrix V;

	if (C.eigen(D, V)) {
		for (unsigned int i = 0; i<D.size(); ++i) {
			D[i] = sqrt(D[i]);
		}

		for (unsigned int i = 0; i<1000; ++i) {
			double xx = GaussRand()/2.;
			double yy = GaussRand()/2.;

			double x = D[0]*xx;
			double y = D[1]*yy;

			double rx = V(0,0)*x + V(0,1)*y;
			double ry = V(1,0)*x + V(1,1)*y;

			dc.SetPixel ((int)(offset_x + rx), (int)(offset_y - ry), RGB(0,0,255));
		}
	}
}

